VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Action"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
'// Holds information about a single action.

Private Const BufferLen = 2048

Private sName As String
Private sPattern As String
Private sCommand As String
Private bEnabled As Boolean
Private bSimple As Boolean
Private oRegExp As RegExp
Private bCaseSensitive As Boolean
Private lSearchFromIdx As Long

Private Function SimplePattern2Regexp(sSimple As String) As String
    SimplePattern2Regexp = "(?:" & sSimple & "|" & sSimple & "\s+([^\n]*))"
End Function

Private Sub UpdatePattern()
    On Error GoTo RegexpSyntaxError
    If Not bEnabled Then Exit Sub
    Dim sTmpPattern As String
    sTmpPattern = sPattern
    If bSimple Then sTmpPattern = Glob2Regexp(sTmpPattern)
    oRegExp.Pattern = ParseHomeMadeRegExp(sTmpPattern)
    oRegExp.Execute "Syntax check"
    lSearchFromIdx = ActionBufferLen + 1
Exit Sub
RegexpSyntaxError:
    oRegExp.Pattern = ""
    bEnabled = False
    Err.Raise Regexp_SyntaxError, "Syntax error in regexp. " & sName & " has been disabled.", TypeName(Me)
End Sub

Public Property Get Name() As String
'// Read/write. The name of the action.
    Name = sName
End Property
Public Property Let Name(sVal As String)
    sName = sVal
End Property

Public Property Get Pattern() As String
'// Read/write. The pattern of the action.
    Pattern = sPattern
End Property

Public Property Let Pattern(sVal As String)
    sPattern = sVal
    UpdatePattern
End Property

Public Property Get Command() As String
'// Read/write. The commands to be executed once the action is triggered.
    Command = sCommand
End Property
Public Property Let Command(sVal As String)
    On Error GoTo errHandler
    sCommand = ConvertToCRLF(sVal)
    lSearchFromIdx = ActionBufferLen + 1
    Exit Property
errHandler:
    ErrorIn "Action.Command(sVal)", sVal
End Property

Public Property Get Enabled() As Boolean
'// Read/write. Enable/disable an action.
    Enabled = bEnabled
End Property
Public Property Let Enabled(bVal As Boolean)
    On Error GoTo errHandler
    If bEnabled = bVal Then Exit Property
    bEnabled = bVal
    UpdatePattern
    Exit Property
errHandler:
    ErrorIn "Action.Enabled(bVal)", bVal
End Property

Public Property Get CaseSensitive() As Boolean
'// Read/write. When True only patterns with the right capitalization are matched.
    On Error GoTo errHandler
    CaseSensitive = Not oRegExp.IgnoreCase
    Exit Property
errHandler:
    ErrorIn "Action.CaseSensitive"
End Property
Public Property Let CaseSensitive(bVal As Boolean)
    oRegExp.IgnoreCase = Not bVal
End Property

Public Property Get Simple() As Boolean
'// Read/write. When True the pattern to match will be parsed as a GLOB pattern, when false a Regular Expression match is made.
    Simple = bSimple
End Property
Public Property Let Simple(bVal As Boolean)
    If bSimple = bVal Then Exit Property
    bSimple = bVal
    UpdatePattern
End Property

Public Property Get RegExp() As VBScript_RegExp_55.RegExp
'// Returns the Regular Expresion match of an action. Even GLOB patterns are transformed to RegExp matches.
    On Error GoTo errHandler
    Set RegExp = oRegExp
    Exit Property
errHandler:
    ErrorIn "Action.RegExp"
End Property

Public Property Get SearchFromIdx() As Long
    SearchFromIdx = lSearchFromIdx
End Property
Public Property Let SearchFromIdx(lVal As Long)
    lSearchFromIdx = lVal
End Property


Private Sub Class_Initialize()
    On Error GoTo errHandler
    Set oRegExp = New VBScript_RegExp_55.RegExp
    oRegExp.MultiLine = True
    oRegExp.Global = False
    bEnabled = True
    bCaseSensitive = False
    oRegExp.IgnoreCase = Not bCaseSensitive
    Exit Sub
errHandler:
    ErrorIn "Action.Class_Initialize"
End Sub

