VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsEvent"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'// This class holds information about a single event. Many properties are readonly, because changing them have drastic effects. If you need to change them, use the add/remove methods available in the EventCollection.
Option Explicit

Private oSession As frmSession

Private blnEnabled As Boolean
Private strName As String
Private strCommand As String
Private udtType As eEventType
Private objData As Collection   ' Store different data, depending on Type.

Friend Sub Init(SessionObject As frmSession)
    Set oSession = SessionObject
End Sub

Friend Sub SetData(Data As Collection)
    If Not Data Is objData Then 'only if nothing
        Set objData = Data
        ForceFullUpdate
    End If
End Sub

Public Function GetData() As Collection
'// Returns a collection (possible Nothing) that holds extra information about an event. See EventCollection descriptions for more information.
    Set GetData = objData
End Function

Friend Property Let EventType(EType As eEventType)
    If EType <> udtType Then
        udtType = EType
        ForceFullUpdate
    End If
End Property

Public Property Get EventType() As eEventType
'// Returns a number indicating what kind of event this object is.
    EventType = udtType
End Property

Public Property Let Name(EventName As String)
    If strName <> EventName Then
        strName = EventName
        ForceFullUpdate
    End If
End Property

Public Property Get Name() As String
'// Returns the name of the event.
    Name = strName
End Property

Public Property Let Enabled(Enable As Boolean)
'// Change the Enabled status of an event. This has no effect for Timer events since they must be enabled/disabled through the EventCollection.
    If blnEnabled <> Enable Then
        blnEnabled = Enable
        ForceFullUpdate
    End If
End Property

Public Property Get Enabled() As Boolean
'// Returns the Enabled status of an event.
    Enabled = blnEnabled
End Property

Public Property Let Command(CommandText As String)
'// Sets the command of an event. This is the action the event takes when it's fired.
    strCommand = CommandText
End Property

Public Property Get Command() As String
'// Returns the command of an event.
    Command = strCommand
End Property

Private Sub Class_Initialize()
    blnEnabled = False
    'blnReadOnly = True
    udtType = etUndefinied
End Sub

Private Sub ForceFullUpdate()
    Dim l As Long
    Dim lIdx As Long
    Dim strOrigName As String
    If oSession Is Nothing Then Exit Sub 'Possibly a new event that hasn't been added to the collection yet?
    With oSession.Events
        'Generate unique name
        strOrigName = strName
        l = 0
        While AnyOtherWithSameName
            l = l + 1
            strName = strOrigName & l
        Wend
        'Remove/add to force the update
        .Remove Me
        .AddObject Me
    End With
End Sub

Private Function AnyOtherWithSameName() As Boolean 'tool to generate unique names
    Dim l As Long
    Dim lIdx As Long
    With oSession.Events
        For l = 1 To .Count
            If Not (.Item(l) Is Me) And (LCase(.Item(l).Name) = LCase(strName)) Then
                AnyOtherWithSameName = True
                Exit Function
            End If
        Next
    End With
    AnyOtherWithSameName = False
End Function
