VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "EventCollection"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
'// This class holds a collection of userdefined events. It it through this class that any changes to events should be made.

Private objEventCol As Collection
Private SessionObject As frmSession

Friend Sub Init(SessionForm As frmSession)
    Set SessionObject = SessionForm
End Sub

Public Sub AddObject(objEvent As clsEvent)
'// Adds an event object to the event collection.
    On Error GoTo errHandler
    
    If (objEvent.EventType = etUndefinied) Then Exit Sub
    
    objEventCol.Add objEvent, objEvent.Name ' EventName has to be unique.
    objEvent.Init SessionObject
    If objEvent.Enabled And (objEvent.EventType = etTimer) Then SessionObject.NewTimer objEvent.Name, objEvent.GetData("Interval"), CBool(objEvent.GetData("FireOnce"))
    
    Exit Sub
errHandler:
    ErrorIn "EventCollection.Add(objEvent)", objEvent
End Sub

Public Sub Remove(Index As Variant)
'// Removes the event with the name/index passed.
On Error GoTo ErrorH
    Dim objEvent As clsEvent
    Dim lIdx As Long
    
    lIdx = GetIndex(Index)
    If lIdx = -1 Then Exit Sub
    Set objEvent = objEventCol(lIdx)
    
    objEventCol.Remove lIdx
    SessionObject.RemoveUnusedTimers

ErrorH:
End Sub

Public Sub Add(EventType As eEventType, EventName As String, Command As String, DataValues As Collection, Optional Enabled As Boolean = True)
'// Adds an event. <b>EventType</b> is defined below. <b>EventName</b> must be a unique name identifying the event.
'// <b>Enabled</b> is default on, but can be called with the value False if the event is not to be enabled yet.
'// <b>DataValues</b> is a collection used for some types of events.
'// Timer-events needs two keys "Interval" and "FireOnce":
'//     <i>Interval</i> is the number of seconds between each fire. The value is a Single type.
'//     <i>FireOnce</i> is a boolean, indicating if the event should be fired more than once.
'// EventType is an Enum with the following values:
'//     etUndefinied = 0 '  <u>DO NOT USE!</u>
'//     etTimer = 1
'//     etWeb = 2
'//     etOnSave = 3
'//     etOnLoad = 4
'//     etOnConnect = 5
'//     etOnDisconnect = 6
'//     etOnOpen = 7
'//     etOnClose = 8
'//     etOnSelectedText = 9
'//     etOnDocumentComplete = 10
'//     etOnTerminalSelect = 11

    On Error GoTo errHandler
    Dim objEvent As clsEvent
    Dim lEventIdx As Long
    
    If (EventType = etUndefinied) Then Exit Sub
    
    lEventIdx = GetIndex(EventName)
    If lEventIdx < 0 Then
        Set objEvent = New clsEvent
    Else
        Set objEvent = objEventCol(lEventIdx)
    End If
    objEvent.Name = EventName
    objEvent.EventType = EventType
    objEvent.Command = Command
    objEvent.SetData DataValues
    objEvent.Enabled = Enabled
    objEventCol.Add objEvent, EventName ' EventName has to be unique.
    objEvent.Init SessionObject
    If objEvent.Enabled And (objEvent.EventType = etTimer) Then SessionObject.NewTimer objEvent.Name, objEvent.GetData("Interval"), CBool(objEvent.GetData("FireOnce"))
    
    
    Exit Sub
errHandler:
    ErrorIn "EventCollection.Add(EventType,EventName,Command,DataValues,Enabled)", Array(EventType, _
         EventName, Command, DataValues, Enabled)
End Sub
'
'Private Sub RemoveTimer(EventName As String)
'    On Error GoTo errHandler
'    SessionObject.RemoveTimer EventName
'    objEventCol.Remove EventName
'    Exit Sub
'errHandler:
'    ErrorIn "EventCollection.RemoveTimer(EventName)", EventName
'End Sub

'Public Sub EnableEvent(EventName As String, Enable As Boolean)
''// Enable or disable an event. If Enable is True, the event will be enabled, otherwise disabled.
'    On Error GoTo errHandler
'    Dim objEvent As clsEvent
'
'    Set objEvent = objEventCol(EventName)
'    If (objEvent Is Nothing) Then Exit Sub
'    objEvent.Enabled = Enable
'    If objEvent.EventType = etTimer Then
'        SessionObject.RemoveUnusedTimers
'        If Enable Then
'            SessionObject.NewTimer objEvent.Name, objEvent.GetData("Interval"), CBool(objEvent.GetData("FireOnce"))
'        End If
'    End If
'    Exit Sub
'errHandler:
'    ErrorIn "EventCollection.EnableEvent(EventName,Enable)", Array(EventName, Enable)
'End Sub

Friend Function FireEvent(EventType As eEventType, Optional EventName As String = "", Optional ParamValues As Variant = Null) As Boolean
    On Error GoTo errHandler
    Dim objEvent As clsEvent
    Dim objMatches As Collection, objEvents As New Collection
    Dim lngLoop As Long
    Dim varEvent As Variant
    Dim lngBuffer As enmBuffer
    Dim blnFired As Boolean
    
    blnFired = False
    For Each varEvent In objEventCol 'Select those that match the specified criteria (and is enabled)
        Set objEvent = varEvent
        If (objEvent.EventType = EventType) And (objEvent.Enabled) Then
            If (EventName = "") Or (objEvent.Name = EventName) Or (objEvent.EventType = etWeb) Then
                Select Case objEvent.EventType
                    Case etWeb
                        Dim objRegExp As RegExp
                        Dim objRMatches As MatchCollection
                        Set objRegExp = New RegExp
                        objRegExp.Pattern = objEvent.GetData("Pattern")
                        Set objRMatches = objRegExp.Execute(ParamValues(0))
                        If objRMatches.Count > 0 Then
                            Set objMatches = New Collection
                            objMatches.Add CStr(ParamValues(0))
                            For lngLoop = 1 To objRMatches(0).SubMatches.Count
                                objMatches.Add CStr(objRMatches(0).SubMatches(lngLoop - 1))
                            Next
                            Set objRMatches = Nothing
                            objEvents.Add objEvent
                        End If
                        Set objRegExp = Nothing
                    Case Else
                        objEvents.Add objEvent
                End Select
            End If
        End If
    Next
    
    If Not IsNull(ParamValues) And (EventType <> etWeb) Then     ' compose match collection
        Set objMatches = New Collection
        objMatches.Add Join(ParamValues, ";") '%0
        For lngLoop = 0 To UBound(ParamValues)
            objMatches.Add CStr(ParamValues(lngLoop))
        Next
    End If
    
    For Each varEvent In objEvents
        Set objEvent = varEvent
        Select Case objEvent.EventType
            Case etTimer
                lngBuffer = bf_MUDResponse
                If objEvent.GetData("FireOnce") Then
                    objEvent.Enabled = False
                End If
            Case etWeb
                lngBuffer = bf_WebResponse
            Case etUndefinied '???
                Exit Function
            Case Else
                lngBuffer = bf_MUDResponse
        End Select
        If lngBuffer = bf_MUDResponse Then
            SessionObject.EvaluateCommand objEvent.Command, objMatches, SessionObject.Actions.GagOutput, True
        Else
            SessionObject.CmdHandler.Execute objEvent.Command, objMatches, lngBuffer, True
        End If
        blnFired = True
    Next
    FireEvent = blnFired
    Exit Function
errHandler:
    ErrorIn "EventCollection.FireEvent(EventName,ParamValues)", Array(EventName, ParamValues)
End Function

Public Function Count()
'// Returns the number of defined events.
    Count = objEventCol.Count
End Function

Friend Function GetIndex(Index) As Long
On Error GoTo ErrorH
    Dim lngLoop As Long
    GetIndex = -1
    Select Case VarType(Index)
        Case vbLong, vbByte, vbInteger
            If Index <= objEventCol.Count Then GetIndex = Index
        Case vbString
            For lngLoop = 1 To objEventCol.Count
                If LCase(objEventCol(lngLoop).Name) = LCase(Index) Then
                    GetIndex = lngLoop
                    Exit Function
                End If
            Next
        Case vbObject
            For lngLoop = 1 To objEventCol.Count
                If objEventCol(lngLoop) Is Index Then
                    GetIndex = lngLoop
                    Exit Function
                End If
            Next
    End Select
    Exit Function
ErrorH:
    GetIndex = -1
End Function

Public Property Get Item(Index As Variant) As clsEvent
Attribute Item.VB_UserMemId = 0
'// Returns an Event object. Index can be a numeric index in the collection, a name of the event or event the object itself ;)
On Error GoTo ErrorH
    
    Set Item = objEventCol(GetIndex(Index))
    Exit Property
ErrorH:
    Set Item = Nothing
End Property

Private Sub Class_Initialize()
    Set objEventCol = New Collection
End Sub
