VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "RegistryHandler"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'Error Constants
    Private Const NonExistingKey = 1 + vbObjectError
    Private Const NonExistingValue = 2 + vbObjectError
    Private Const TypeMismatch = 3 + vbObjectError 'raises if the key isn't a string value
    Private Const CreationError = 4 + vbObjectError

'registry API constants
    Private Const REG_SZ As Long = 1    'zero terminated string pointer type
    Private Const REG_BINARY& = 3
    Private Const REG_DWORD& = 4
    Private Const REG_DWORD_BIG_ENDIAN& = 5
    Private Const REG_DWORD_LITTLE_ENDIAN& = 4
    Private Const REG_EXPAND_SZ& = 2
    Private Const REG_LINK& = 6
    Private Const REG_MULTI_SZ& = 7
    Private Const REG_NONE& = 0

Public Enum TopKey
    HKEY_CLASSES_ROOT = &H80000000
    HKEY_CURRENT_USER = &H80000001
    HKEY_LOCAL_MACHINE = &H80000002
    HKEY_USERS = &H80000003
End Enum

Private Type SECURITY_ATTRIBUTES
        nLength As Long
        lpSecurityDescriptor As Long
        bInheritHandle As Long
End Type

    Private Const ERROR_NONE = 0

    Private Const STANDARD_RIGHTS_ALL = &H1F0000
    Private Const STANDARD_RIGHTS_EXECUTE = &H20000
    Private Const STANDARD_RIGHTS_READ& = &H20000
    Private Const STANDARD_RIGHTS_REQUIRED = &HF0000
    Private Const STANDARD_RIGHTS_WRITE& = &H20000
    Private Const SYNCHRONIZE = &H100000


    Private Const KEY_ALL_ACCESS& = &H3F
    Private Const KEY_QUERY_VALUE& = &H1
    Private Const KEY_SET_VALUE& = &H2
    Private Const KEY_CREATE_SUB_KEY = &H4
    Private Const KEY_WRITE = ((STANDARD_RIGHTS_WRITE Or _
        KEY_SET_VALUE Or KEY_CREATE_SUB_KEY) And _
        (Not SYNCHRONIZE))

    Private Const REG_OPTION_NON_VOLATILE = 0       ' Key is preserved when system is rebooted
    Private Const REG_CREATED_NEW_KEY = &H1                      ' New Registry Key created
    Private Const REG_OPENED_EXISTING_KEY = &H2                      ' Existing Key opened


'registry API functions
    Private Declare Function RegCloseKey Lib "advapi32.dll" _
        (ByVal hKey As Long) As Long
    Private Declare Function RegOpenKeyEx Lib "advapi32.dll" Alias _
        "RegOpenKeyExA" (ByVal hKey As Long, ByVal lpSubKey As String, _
        ByVal ulOptions As Long, ByVal samDesired As Long, phkResult As _
        Long) As Long
    Private Declare Function RegOpenRootKeyEx Lib "advapi32.dll" Alias _
        "RegOpenKeyExA" (ByVal hKey As Long, ByVal lpSubKey As Long, _
        ByVal ulOptions As Long, ByVal samDesired As Long, phkResult As _
        Long) As Long
    Private Declare Function RegQueryValueEx Lib "advapi32.dll" Alias _
        "RegQueryValueExA" (ByVal hKey As Long, ByVal lpValueName As _
        String, ByVal lpReserved As Long, lpType As Long, ByVal lpData _
        As Any, lpcbData As Long) As Long
    Private Declare Function RegSetValueEx Lib "advapi32.dll" Alias _
        "RegSetValueExA" (ByVal hKey As Long, ByVal lpValueName As String, _
        ByVal Reserved As Long, ByVal dwType As Long, ByRef lpData As Any, _
        ByVal cbData As Long) As Long
    Private Declare Function RegCreateKeyEx Lib "advapi32.dll" Alias _
        "RegCreateKeyExA" (ByVal hKey As Long, ByVal lpSubKey As String, _
        ByVal Reserved As Long, ByVal lpClass As String, ByVal dwOptions _
        As Long, ByVal samDesired As Long, lpSecurityAttributes As _
        SECURITY_ATTRIBUTES, phkResult As Long, lpdwDisposition As Long) As Long
    Private Declare Function RegCreateKey Lib "advapi32.dll" Alias _
        "RegCreateKeyA" (ByVal hKey As Long, ByVal lpSubKey As String, phkResult As Long) As Long
Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (Destination As Any, Source As Any, ByVal Length As Long)
Private Declare Function ExpandEnvironmentStrings Lib "kernel32" Alias "ExpandEnvironmentStringsA" (ByVal lpSrc As String, ByVal lpDst As String, ByVal nSize As Long) As Long




Public Function ExistsKey(Root As TopKey, ByVal sKeyName) As Boolean
    On Error GoTo errHandler
    Dim hKey As Long
    ExistsKey = (RegOpenKeyEx(Root, sKeyName, 0, KEY_QUERY_VALUE, hKey) = ERROR_NONE)
    Exit Function
errHandler:
    ErrorIn "RegistryHandler.ExistsKey(Root,sKeyName)", Array(Root, sKeyName)
End Function
        
Public Function GetSetting(Root As TopKey, ByVal sKeyName, ByVal szValueName As String) As Variant
    On Error GoTo errHandler
'gets a string setting from the registry.
'Raises an error if the key or the value doesn't exist
    Dim hKey As Long
    Dim cch As Long
    Dim lType As Long
    Dim sValue As String
    Dim strTemp As String
    Dim lvalue As Long
    Dim tRes As Long
    Dim lngLP As Long
    Dim lng As Long
    tRes = RegOpenKeyEx(Root, sKeyName, 0, KEY_QUERY_VALUE, hKey)
    If tRes <> ERROR_NONE Then
        Err.Raise NonExistingKey, "RegistryHandler.GetSetting", _
            "Non Existing Registry key: " & sKeyName
    End If
    tRes = RegQueryValueEx(hKey, szValueName, 0&, lType, 0&, cch)
    'can be omitted if cch is set to a max length.
    If tRes <> ERROR_NONE Then
        Err.Raise NonExistingValue, "RegistryHandler.GetSetting", _
            "Non Existing Registry Value: " & szValueName
    End If
    Select Case lType
        Case REG_SZ
            sValue = String(cch, 0)
            tRes = RegQueryValueEx(hKey, szValueName, 0&, lType, sValue, cch)
            'don't check for errors because we've already read from that value
            GetSetting = Left$(sValue, cch - 1) 'remove the zero
        Case REG_EXPAND_SZ
            sValue = String(cch, 0)
            tRes = RegQueryValueEx(hKey, szValueName, 0&, lType, sValue, cch)
            'don't check for errors because we've already read from that value
            tRes = ExpandEnvironmentStrings(sValue, strTemp, 0)
            strTemp = Space(tRes)
            tRes = ExpandEnvironmentStrings(sValue, strTemp, tRes)
            GetSetting = Left$(strTemp, tRes - 1) 'remove the zero
        Case REG_MULTI_SZ
            sValue = String(cch, 0)
            tRes = RegQueryValueEx(hKey, szValueName, 0&, lType, sValue, cch)
            'don't check for errors because we've already read from that value
            GetSetting = Split(Left$(sValue, cch - 1), vbNullChar) 'remove the zero
        Case REG_DWORD, REG_DWORD_LITTLE_ENDIAN
            tRes = RegQueryValueEx(hKey, szValueName, 0&, lType, VarPtr(lng), 4)
            GetSetting = lng
        Case REG_DWORD_BIG_ENDIAN
            tRes = RegQueryValueEx(hKey, szValueName, 0&, lType, VarPtr(lngLP), 4)
            Call CopyMemory(VarPtr(lng) + 2&, VarPtr(lngLP), 2)
            Call CopyMemory(VarPtr(lng), VarPtr(lngLP) + 2&, 2)
            GetSetting = lng
        Case REG_BINARY
            Dim bytArr() As Byte
            ReDim bytArr(cch - 1)
            tRes = RegQueryValueEx(hKey, szValueName, 0&, lType, VarPtr(bytArr(0)), cch)
            GetSetting = bytArr
        Case Else
            Err.Raise TypeMismatch, "RegistryHandler.GetSetting", _
                "UnKnown data type."
    End Select
    tRes = RegCloseKey(hKey)
    Exit Function
errHandler:
    ErrorIn "RegistryHandler.GetSetting(Root,sKeyName,szValueName)", Array(Root, sKeyName, _
         szValueName)
End Function

Public Sub SetSetting(Root As TopKey, ByVal sKeyName, ByVal szValueName As String, Value As Variant)
    On Error GoTo errHandler
'gets a string setting from the registry.
'Raises an error if the key or the value doesn't exist
    Dim hKey As Long
    Dim cch As Long
    Dim lType As Long
    Dim lvalue As Long
    Dim sValue As String
    Dim BArray() As Byte
    Dim L As Long
    Dim tRes As Long
    tRes = RegOpenKeyEx(Root, sKeyName, 0, KEY_SET_VALUE, hKey)
    If tRes <> ERROR_NONE Then
        Err.Raise NonExistingKey, "RegistryHandler.GetSetting", _
            "Non Existing Registry key: " & sKeyName
    End If
    'tRes = RegQueryValueEx(hKey, szValueName, 0&, lType, 0&, cch)
    'can be omitted if cch is set to a max length.
    'If tRes <> ERROR_NONE Then
    '    Err.Raise NonExistingValue, "RegistryHandler.GetSetting", _
    '        "Non Existing Registry Value: " & szValueName
    'End If
    Select Case VarType(Value)
        Case vbString
            lType = REG_SZ
            sValue = CStr(Value) & vbNullChar
            ReDim BArray(Len(sValue))
            For L = 1 To Len(sValue)
                BArray(L - 1) = Asc(Mid$(sValue, L, 1))
            Next
            tRes = RegSetValueEx(hKey, szValueName, 0&, lType, BArray(0), Len(sValue))
            'don't check for errors because we've already read from that value
        Case vbInteger, vbLong, vbByte
            lType = REG_DWORD
            lvalue = Value
            tRes = RegSetValueEx(hKey, szValueName, 0&, lType, lvalue, 4)
        Case vbArray + vbString
            lType = REG_MULTI_SZ
            sValue = Join(Value, vbNullChar) & vbNullChar
            ReDim BArray(Len(sValue))
            For L = 1 To Len(sValue)
                BArray(L - 1) = Asc(Mid$(sValue, L, 1))
            Next
            tRes = RegSetValueEx(hKey, szValueName, 0&, lType, BArray(0), Len(sValue))
        Case vbArray + vbByte
            lType = REG_BINARY
            BArray = Value
            tRes = RegSetValueEx(hKey, szValueName, 0&, lType, BArray(0), UBound(Value) + 1)
        Case Else
            Err.Raise TypeMismatch, "RegistryHandler.GetSetting", _
                "UnKnown data type."
    End Select
    tRes = RegCloseKey(hKey)
    Exit Sub
errHandler:
    ErrorIn "RegistryHandler.SetSetting(Root,sKeyName,szValueName,Value)", Array(Root, sKeyName, _
         szValueName, Value)
End Sub


Public Sub CreateKey(Root As TopKey, ByVal sKeyName As String)
    On Error GoTo errHandler
'gets a string setting from the registry.
'Raises an error if the key or the value doesn't exist
    Dim hKey As Long
    Dim cch As Long
    Dim lType As Long
    Dim sValue As String
    Dim lvalue As Long
    Dim tRes As Long
    Dim tDisp As Long
    Dim SecAttr As SECURITY_ATTRIBUTES  ' security settings of the key
    
    SecAttr.nLength = Len(SecAttr)  ' size of the structure
    SecAttr.lpSecurityDescriptor = 0  ' default security level
    SecAttr.bInheritHandle = True  ' the default value for this setting

    'tRes = RegOpenKeyEx(Root, "software", 0, KEY_WRITE, hKey)
    'If tRes <> ERROR_NONE Then
    '    Err.Raise NonExistingKey, "RegistryHandler.GetSetting", _
    '        "Non Existing Registry key: Root?"
    'End If
    
    tRes = RegCreateKeyEx(Root, sKeyName, 0, vbNullString, REG_OPTION_NON_VOLATILE, KEY_WRITE, _
                 SecAttr, hKey, tDisp)
    'can be omitted if cch is set to a max length.
    If tRes <> ERROR_NONE Then
        Err.Raise CreationError, "RegistryHandler.CreateKey", _
            "Couldn't create key: " & sKeyName
    End If
    tRes = RegCloseKey(hKey)
    Exit Sub
errHandler:
    ErrorIn "RegistryHandler.CreateKey(Root,sKeyName)", Array(Root, sKeyName)
End Sub
