VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "VT100Parser"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
'// This object is responsible for translating the incomming and outgoing telnet commands and ANSI codes.

Private Enum TelnetCommandEnum
    ' http://www.scit.wlv.ac.uk/~jphb/comms/telnet.html
    ' http://students.cs.byu.edu/~yamadah/Telnet.html
    tce_IS = 0
    tce_SEND = 1
    tce_INFO = 2
    '------------
    tce_Var = 0
    tce_Value = 1
    tce_ESC = 2
    tce_USERVAR = 3
    '--------------
    tce_OFF = 0
    tce_ON = 1
    tce_RESTARTANY = 2
    tce_RESTARTXON = 3

    '--------------
    tce_SE = 240
    tce_NOP = 241
    tce_DataMark = 242
    tce_Break = 243
    tce_InterruptProcess = 244
    tce_AbortOutput = 245
    tce_AreYouThere = 246
    tce_EraseCharacter = 247
    tce_EraseLine = 248
    tce_GA = 249
    tce_SB = 250
    tce_WILL = 251
    tce_WONT = 252
    tce_DO = 253
    tce_DONT = 254
    tce_IAC = 255
    '-------------
    tce_Echo = 1 'RFC
    tce_SuppresGoAhead = 3
    tce_Status = 5
    tce_TimingMark = 6
    tce_TerminalType = 24 'rfc1091
    tce_WindowSize = 31 'rfc1073
    tce_TerminalSpeed = 32
    tce_RemoteFlowControl = 33
    tce_LineMode = 34
    tce_XDisplayLocation = 35
    tce_EnvironmentVariables = 36
    tce_NewEnviron = 39
End Enum

Private bSuppressGoAhead As Boolean 'only used to give consistent replies on tce_status

Private Const AnsiBufferSize = &H3FFF

Private TXTLogInfo As typLogInfo
Private WithEvents Terminal As Terminal
Attribute Terminal.VB_VarHelpID = -1
Private ANSILogInfo As typLogInfo

Private arrTerminalTypes() As String
Private lTerminalType As Long

Private Enum AnsiStateEnum
    ase_Text = 0
    ase_Escape = 1
    ase_Command = 2
    ase_Telnet_IAC = 10 'used for turning echo on and off
End Enum

Private Type BufferType
    Color As Byte 'high 4bit is backcolor
    Char As Byte
End Type

Private SavedCursor As New Cursor

Private arrAnsiBuffer(AnsiBufferSize) As Byte
Private AnsiBufferStart As Long
Private AnsiBufferEnd As Long

Public DefaultColor As Byte
'// Defines the colorindex to be used for normal text. Default value is 7, and default color of index 7 is gray.
Public TabWidth As Long
'// Defines the tab-width. Default value is 8 spaces.
Public GagEmptyLines As Boolean
'// When set, all empty lines will be gagged (not printet on the terminal).

Private Type PositionType
    Xpos As Long
    YPos As Long
End Type

Public Event TelnetReply(strCommandSequence As String)

Private Sub SaveCursor()
    With Terminal.Cursor
        SavedCursor.Color = .Color
        SavedCursor.Xpos = .Xpos
        SavedCursor.YPos = .YPos
    End With
End Sub

Friend Sub Init(oTerminal As Terminal)
    Set Terminal = oTerminal
    Terminal.Cursor.Color = DefaultColor
    SaveCursor
End Sub

Private Sub Class_Initialize()
    On Error GoTo errHandler
    arrTerminalTypes = Split("ANSI,VT100,DEC-VT100,DEC-VT100") 'last terminal type should be repeated
    lTerminalType = 0

    DefaultColor = 7
     
    Randomize
    AnsiBufferStart = 0
    AnsiBufferEnd = 0
    Exit Sub
errHandler:
    ErrorIn "VT100Parser.Class_Initialize"
End Sub

Private Sub SetCurColor(strArgs As String)
    'm if no args -> ResetColors
    '     args: 0,1 HighLight Off,On
    '     args: 4,5,7,8 UL(mono),Blink,ReverseVideo,Invisible
    '     args: 30-37: foreground, 40-47: BackGround
    Dim Arg() As String
    Dim lngArg As Long
    Dim lng As Long
    Dim AColorWasSet As Boolean
    On Error Resume Next
    Arg = Split(strArgs, ";")
    AColorWasSet = False
    With Terminal.Cursor
        If strArgs = "" Then
            .Color = .Color And (&H77)
            Exit Sub
        End If
        For lng = 0 To UBound(Arg)
            If Len(Arg(lng)) = 0 Then
                .Color = .Color And (&H77) 'Remove Bold and ReverseVideo
            Else
                lngArg = CLng(Arg(lng))
                Select Case lngArg
                    Case 0
	                .Color = DefaultColor
                    Case 1
                        .Color = .Color Or &H8
                    Case 2
                    	.Color = .Color And &HF7
                    Case 7
                        .Color = .Color Xor &H80
                    Case 30 To 37
                        .Color = (.Color And &HF8) Or (lngArg - 30)
                    Case 40 To 47
                        .Color = (.Color And &H8F) Or ((lngArg - 40) * &H10)
                    Case 999 'Recall cursor colors
                        .Color = SavedCursor.Color
                    Case Else '4,5,8
                End Select
            End If
        Next
    End With
End Sub

Private Sub SetCursorPos(strArgs As String, C As String)
    On Error GoTo errHandler
    'A: CursorUp (arg optional)
    'B: CursorDown (arg optional)
    'C: CursorRight (arg optional)
    'D: CursorLeft (arg optional)
    'H,f: Set Cursorpos (args: row, col)
    Dim Arg() As String
    Dim lngArg As Long, lngArg2 As Long
    Dim lngDY As Long
    Arg() = Split(strArgs, ";")
    If strArgs = "" Then
        lngArg = 1
    Else
        On Error Resume Next
        lngArg = CLng(Arg(0))
        lngArg2 = CLng(Arg(1))
        On Error GoTo errHandler
    End If
    With Terminal.Cursor
        Select Case C
            Case "A"
                .YPos = .YPos - lngArg
            Case "B"
                .YPos = .YPos + lngArg
            Case "C"
                .Xpos = (.Xpos - lngArg) Mod (Terminal.PageWidth + 1)
                .YPos = .YPos - (lngArg - .Xpos - Terminal.PageWidth) \ (Terminal.PageWidth + 1)
                If .Xpos < 0 Then .Xpos = .Xpos + Terminal.PageWidth + 1
            Case "D"
                .Xpos = (.Xpos + lngArg) Mod (Terminal.PageWidth + 1)
                .YPos = .YPos + (.Xpos + lngArg) \ (Terminal.PageWidth + 1)
            Case "H", "f"
                If (UBound(Arg) = 1) And (lngArg > 0) And (lngArg2 > 0) Then
                    .Xpos = (lngArg2 - 1) Mod Terminal.PageWidth
                    .YPos = lngArg - 1 + (lngArg2 - 1) \ (Terminal.PageWidth + 1)
                Else ' reset to top left corner.
                    .Xpos = 0
                    .YPos = 0
                End If
            Case "s"
                SaveCursor
            Case "u"
                .Xpos = SavedCursor.Xpos
                .YPos = SavedCursor.YPos
        End Select
        If .YPos < 0 Then
            .YPos = 0
        ElseIf .YPos > Terminal.PageHeight Then
            .YPos = Terminal.PageHeight
        End If
    End With
    Exit Sub
errHandler:
    ErrorIn "VT100Parser.SetCursorPos(strArgs,c)", Array(strArgs, C)
End Sub

Private Sub ParseEraseAnsi(strArgs As String, C As Byte)
    On Error GoTo errHandler
    Dim lngX As Long, lngY As Long
    Dim lX1 As Long, lX2 As Long, lY1 As Long, lY2 As Long
    Dim ltX1 As Long, ltX2 As Long
    Dim BufferY As Long
    Select Case Chr(C)
        Case "J", "K"
            lX1 = 0
            lX2 = Terminal.PageWidth
            If Chr(C) = "K" Then
                lY1 = Terminal.Cursor.YPos
                lY2 = Terminal.Cursor.YPos
            Else
                lY1 = 0
                lY2 = Terminal.PageHeight
            End If
            Select Case strArgs
                Case "": lX1 = Terminal.Cursor.Xpos: lY1 = Terminal.Cursor.YPos
                Case "1": lX2 = Terminal.Cursor.Xpos: lY2 = Terminal.Cursor.YPos
            End Select
            Terminal.FillArea lX1, lY1, lX2, lY2
        Case Else
    End Select
    Exit Sub
errHandler:
    ErrorIn "VT100Parser.ParseEraseAnsi(strArgs,c)", Array(strArgs, C)
End Sub

Private Sub PrintChar(ByRef strStrippedText As String)
    On Error GoTo errHandler
    Dim C As Byte
    Dim lngSpaceCount As Long
    C = arrAnsiBuffer(AnsiBufferStart)
    With Terminal.Cursor
        'Debug.Print Chr(c)
        Select Case C
            Case 10
                If Not (GagEmptyLines And .Xpos = 0) Then Terminal.PrintReturn
                strStrippedText = strStrippedText & Chr(C)
            Case 13, 1 'invisible
            Case 9
                strStrippedText = strStrippedText & Chr(9)
                If (TabWidth > 0) And (TabWidth < Terminal.PageWidth) Then
                    Terminal.PrintAChar 32
                    While Not (((.Xpos + 1) Mod (TabWidth + 1)) = 0)
                        Terminal.PrintAChar 32
                    Wend
                End If
            Case Else
                Terminal.PrintAChar C
                strStrippedText = strStrippedText & Chr(C)
        End Select
    End With
    AnsiBufferStart = (AnsiBufferStart + 1) And AnsiBufferSize
    Exit Sub
errHandler:
    ErrorIn "VT100.PrintChar(strStrippedText)", strStrippedText
End Sub


Private Sub ParseAnsi(ByRef strStrippedText As String)
    On Error GoTo errHandler
    Dim lng As Long
    Dim AnsiState As AnsiStateEnum
    Dim C As Byte
    Dim strArguments As String
    Dim lngX As Long, lngY As Long
    Dim BufferY As Long
    lng = AnsiBufferStart
    AnsiState = ase_Text
    While lng <> AnsiBufferEnd
        C = arrAnsiBuffer(lng)
        Select Case AnsiState
            Case ase_Text
                Select Case C
                    Case 0
                        AnsiBufferStart = (AnsiBufferStart + 1) And AnsiBufferSize
                    Case 7
                        Beep
                        AnsiBufferStart = (AnsiBufferStart + 1) And AnsiBufferSize
                    Case 8
                        With Terminal.Cursor
                            If .Xpos > 0 Then .Xpos = .Xpos - 1
                            Terminal.WriteScreenBuffer .Xpos, .YPos, 32, .Color
                            AnsiBufferStart = (AnsiBufferStart + 1) And AnsiBufferSize
                        End With
                    Case 27 'Escape
                        AnsiState = ase_Escape
                    Case 255
                        AnsiState = ase_Telnet_IAC
                    Case Else
                        PrintChar strStrippedText
                End Select
            Case ase_Escape
                Select Case Chr(C)
                    Case "["
                        AnsiState = ase_Command
                        strArguments = ""
                    'Case "M" scrolldown
                    'Case "D" scrollup
                    Case Else
                        AnsiState = ase_Text
                        PrintChar strStrippedText
                End Select
            Case ase_Command
                Select Case Chr(C)
                    Case "0" To "9", ";"
                        strArguments = strArguments & Chr(C)
                    Case "m"
                        SetCurColor strArguments
                        AnsiState = ase_Text
                    Case "H", "f", "A", "B", "C", "D", "s", "u"
                        SetCursorPos strArguments, Chr(C)
                        AnsiState = ase_Text
                    Case "J", "K"
                        ParseEraseAnsi strArguments, C
                        AnsiState = ase_Text
                    Case "r" 'set window lines
                    Case "n" 'report position
                        If strArguments = "6" Then RaiseEvent TelnetReply(Chr(27) & "[" & Terminal.Cursor.Xpos + 1 & ";" & Terminal.Cursor.YPos + 1 & "R")
                    Case Else
                        AnsiState = ase_Text
                End Select
                If AnsiState = ase_Text Then
                    AnsiBufferStart = (AnsiBufferStart + Len(strArguments) + 3) And AnsiBufferSize
                End If
            Case ase_Telnet_IAC
                strArguments = strArguments & Chr(C)
                If ParseIAC(strArguments) Then
                    AnsiState = ase_Text
                    AnsiBufferStart = (AnsiBufferStart + Len(strArguments) + 1) And AnsiBufferSize
                    strArguments = ""
                End If
        End Select
        'AnsiCommands: esc[arg;arg;arg;...commandspecifier
        'commandspecifier
        'J  Erase from Cursor to end of screen
        '1J Erase from Beginning of screen to cursor   Esc [ 1 J
        '2J       Erase Entire screen                  Esc [ 2 J
        'K  Erase from Cursor to end of line           Esc [ K
        '1K Erase from Beginning of line to cursor     Esc [ 1 K
        '2K Erase Entire line                          Esc [ 2 K
        'L  Insert Line
        'M  Delete line
        'P  Delete Char
        'r  set scrolling region ????
        'g  clear tabs? arg can be 3?
        '?  not implemented
        lng = (lng + 1) And AnsiBufferSize
    Wend
    Terminal.Redraw
    Exit Sub
errHandler:
    ErrorIn "VT100Parser.ParseAnsi(strStrippedText)", strStrippedText
End Sub

Public Property Get LocalEcho() As Boolean
'// Reads or writes the value of the local echo. When set all typed text in the terminal window will show, turning it off can protect against someone snooping you typing your password etc.
    LocalEcho = Terminal.Cursor.Echo
End Property

Public Property Let LocalEcho(bEcho As Boolean)
    Terminal.Cursor.Echo = bEcho
End Property

Public Function WriteString(ByVal Ansi As String) As String
'// Puts the string onto the terminal window and into the log files. The string is first parsed for ANSI codes.
    On Error GoTo errHandler
    Dim lng As Long
    If ANSILogInfo.blnLog Then ANSILogInfo.objFile.Write Ansi
    For lng = 1 To Len(Ansi)
        arrAnsiBuffer((AnsiBufferEnd + lng - 1) And AnsiBufferSize) = Asc(Mid(Ansi, lng, 1))
    Next
    AnsiBufferEnd = (AnsiBufferEnd + Len(Ansi)) And AnsiBufferSize
    WriteString = ""
    ParseAnsi WriteString
    If TXTLogInfo.blnLog Then TXTLogInfo.objFile.Write WriteString
    Exit Function
errHandler:
    ErrorIn "VT100Parser.WriteString(Ansi)", Ansi
End Function

Private Sub Scrollbar_Change()
    On Error GoTo errHandler
    Terminal.Redraw
    Exit Sub
errHandler:
    ErrorIn "VT100Parser.Scrollbar_Change"
End Sub

Private Function NegotiateTerminalSize()
    RaiseEvent TelnetReply(Chr(tce_IAC) & Chr(tce_WILL) & Chr(tce_WindowSize))
End Function

Private Function SendBreak()
    RaiseEvent TelnetReply(Chr(tce_IAC) & Chr(tce_DO) & Chr(tce_Break))
End Function

Private Function ParseIAC(strArguments As String) As Boolean
    On Error GoTo errHandler
    'returns true when strArguments is a complete sequence
    Dim TypeOfOperation As Byte
    Dim Operation As Byte
    Dim strReply As String
    strReply = ""
    If Len(strArguments) > 10 Then ParseIAC = True 'crash protect.... IAC commands should not be longer than 10, atleast not the ones we understand....
    TypeOfOperation = Asc(Mid(strArguments, 1, 1))
    Select Case TypeOfOperation
        Case tce_WILL, tce_WONT, tce_DO, tce_DONT
            If Len(strArguments) = 2 Then
                'DumpString "SERVER", Chr(255) & strArguments
                ParseIAC = True
                Operation = Asc(Mid(strArguments, 2, 1))
                Select Case Operation
                    Case tce_Echo
                        Terminal.Cursor.Echo = (TypeOfOperation = tce_DO) Or (TypeOfOperation = tce_WONT)
                        If TypeOfOperation = tce_DO Then strReply = Chr(tce_IAC) & Chr(tce_WILL) & Chr(Operation)
                        If TypeOfOperation = tce_DONT Then strReply = Chr(tce_IAC) & Chr(tce_WONT) & Chr(Operation)
                        If TypeOfOperation = tce_WILL Then strReply = Chr(tce_IAC) & Chr(tce_DO) & Chr(Operation)
                    Case tce_SuppresGoAhead 'IGNORE....
                        If TypeOfOperation = tce_DO Then bSuppressGoAhead = True
                        If TypeOfOperation = tce_DONT Then bSuppressGoAhead = False
                        If TypeOfOperation = tce_WILL Then strReply = Chr(tce_IAC) & Chr(tce_DO) & Chr(Operation)
                    Case tce_Status
                        If TypeOfOperation = tce_DO Then
                            strReply = Chr(tce_IAC) & Chr(tce_WILL) & Chr(Operation) & _
                                Chr(tce_IAC) & Chr(tce_SB) & Chr(Operation)
                            If Terminal.Cursor.Echo Then
                                strReply = strReply & Chr(tce_WILL) & Chr(tce_Echo)
                            Else
                                strReply = strReply & Chr(tce_WONT) & Chr(tce_Echo)
                            End If
                            If bSuppressGoAhead Then
                                strReply = strReply & Chr(tce_DO) & Chr(tce_SuppresGoAhead)
                            Else
                                strReply = strReply & Chr(tce_DONT) & Chr(tce_SuppresGoAhead)
                            End If
                            strReply = strReply & Chr(tce_IAC) & Chr(tce_SE)
                        End If
                    'Case tce_TimingMark
                    Case tce_TerminalType, tce_TerminalSpeed
                        If TypeOfOperation = tce_DO Then
                            lTerminalType = 0
                            strReply = Chr(tce_IAC) & Chr(tce_WILL) & Chr(Operation)
                        End If
                    Case tce_WindowSize
                        If (TypeOfOperation = tce_DO) Then
                            strReply = Chr(tce_IAC) & Chr(tce_WILL) & Chr(Operation) _
                            & Chr(tce_IAC) & Chr(tce_SB) & Chr(Operation) & Chr(Terminal.PageWidth \ 256) _
                            & Chr(Terminal.PageWidth And 255) & Chr(Terminal.PageHeight \ 256) _
                            & Chr(Terminal.PageHeight And 255) & Chr(tce_IAC) & Chr(tce_SE)
                        End If
                    Case tce_RemoteFlowControl
                        If TypeOfOperation = tce_DO Then
                            strReply = Chr(tce_IAC) & Chr(tce_WONT) & Chr(Operation)
                        End If
                    'Case tce_LineMode
                    'Case tce_EnvironmentVariables
                    'Case tce_NewEnviron 'ignore
                    Case tce_NewEnviron, tce_XDisplayLocation 'Drop these
                        If TypeOfOperation = tce_DO Then
                            strReply = Chr(tce_IAC) & Chr(tce_WILL) & Chr(Operation)
                        End If
                    Case Else
                        'just reply WONT
                        If TypeOfOperation = tce_DO Then
                            strReply = Chr(tce_IAC) & Chr(tce_WONT) & Chr(Operation)
                        End If
                End Select
                ParseIAC = True
            End If
        Case tce_SB 'SUB NEGOTIATION
            If Right(strArguments, 2) = Chr(tce_IAC) & Chr(tce_SE) Then
                Operation = Asc(Mid(strArguments, 2, 1))
                'DumpString "SERVER", strArguments
                Select Case Operation
                    Case tce_TerminalType
                        If Asc(Mid(strArguments, 3, 1)) = tce_SEND Then
                            strReply = Chr(tce_IAC) & Chr(tce_SB) & Chr(Operation) & Chr(tce_IS) & arrTerminalTypes(lTerminalType) & Chr(tce_IAC) & Chr(tce_SE)
                            lTerminalType = lTerminalType + 1
                            If lTerminalType > UBound(arrTerminalTypes) Then lTerminalType = 0
                        End If
                    Case tce_TerminalSpeed
                        If Asc(Mid(strArguments, 3, 1)) = tce_SEND Then
                            strReply = Chr(tce_IAC) & Chr(tce_SB) & Chr(Operation) & Chr(tce_IS) & "28800,28800" & Chr(tce_IAC) & Chr(tce_SE)
                        End If
                    Case tce_NewEnviron, tce_XDisplayLocation
                        If Asc(Mid(strArguments, 3, 1)) = tce_SEND Then
                            strReply = Chr(tce_IAC) & Chr(tce_SB) & Chr(Operation) & Chr(tce_IS) & Chr(tce_IAC) & Chr(tce_SE)
                        End If
                    Case tce_WindowSize
                        If Asc(Mid(strArguments, 3, 1)) = tce_SEND Then
                            strReply = Chr(tce_IAC) & Chr(tce_SB) & Chr(Operation) & Chr(tce_IS) & Chr(Terminal.PageWidth \ 256) & Chr(Terminal.PageWidth And 255) & Chr(Terminal.PageHeight \ 256) & Chr(Terminal.PageHeight And 255) & Chr(tce_IAC) & Chr(tce_SE)
                        End If
                    Case Else
                        'NOT IMPLEMENTED YET
                        'Don't know what to do... just write the whole string to the screen (except the IAC)
                        Terminal.PrintString strArguments, False
                        ParseIAC = True
                End Select
                ParseIAC = True
            End If
        Case tce_GA, tce_NOP: ParseIAC = True 'just ignore go aheads
        Case tce_EraseLine, tce_EraseCharacter, tce_AbortOutput, _
            tce_InterruptProcess, tce_Break, tce_DataMark
            'TODO
            ParseIAC = True
        Case tce_AreYouThere
            strReply = Chr(tce_IAC) & Chr(tce_NOP) 'yes i'm here
            ParseIAC = True
        Case tce_IAC
            PrintChar Chr(255)
            ParseIAC = True
        Case Else
            'Don't know what to do... just write the whole string to the screen (except the IAC)
            Terminal.PrintString strArguments, False
            ParseIAC = True
    End Select
    If strReply <> "" Then
        'DumpString "CLIENT", strReply
        RaiseEvent TelnetReply(strReply)
    End If
    Exit Function
errHandler:
    ErrorIn "VT100Parser.ParseIAC(strArguments)", strArguments
End Function

Private Sub DumpString(Prefix As String, strArg As String)
    On Error GoTo errHandler
    Dim strOut As String
    Dim l As Long
    For l = 1 To Len(strArg)
        strOut = strOut & Right("  " & Asc(Mid(strArg, l, 1)), 3) & " "
    Next
    Debug.Print Prefix & "  " & strOut
    Exit Sub
errHandler:
    ErrorIn "VT100Parser.DumpString(Prefix,strArg)", Array(Prefix, strArg)
End Sub

Public Sub SetLogInfo(Optional FileName As String, Optional StartLog As Boolean = True, Optional WithANSI As Boolean = False)
'// Starts or stop logging. Use this sub with care, as it's not 100% foolproof when calling directly (if it looks like it's working then it probably is - but some combinations may cause errors).
'// Use the menus instead to ensure safe logging.
    If StartLog And ((WithANSI And ANSILogInfo.blnLog) _
    Or ((Not WithANSI) And TXTLogInfo.blnLog)) _
    Then Exit Sub ' Already logging. - Perhaps permit to change filename?
    
    If IsMissing(FileName) Then ' Stop logging
        StartLog = False
    End If
    
    If WithANSI Then
        If StartLog Then
            ANSILogInfo.strFileName = FileName
            Set ANSILogInfo.objFile = GetFileObject(FileName)
        Else
            CloseFileObject ANSILogInfo.objFile
            Set ANSILogInfo.objFile = Nothing
        End If
        ANSILogInfo.blnLog = StartLog
    Else
        If StartLog Then
            TXTLogInfo.strFileName = FileName
            Set TXTLogInfo.objFile = GetFileObject(FileName)
        Else
            CloseFileObject TXTLogInfo.objFile
            Set TXTLogInfo.objFile = Nothing
        End If
        TXTLogInfo.blnLog = StartLog
    End If
End Sub


Public Function IsLogging(Optional WithANSI As Boolean = False) As Boolean
'// Will return a boolean value depending on the current log-status.
    If WithANSI Then IsLogging = ANSILogInfo.blnLog Else IsLogging = TXTLogInfo.blnLog
End Function

Private Sub Terminal_KeyDown(KeyCode As Integer, Shift As Integer)
    Dim C As Byte
    If Shift = 2 Then
        Select Case KeyCode
            Case vbKeyA To vbKeyZ
                RaiseEvent TelnetReply(Chr(KeyCode - vbKeyA + 1))
            Case Else
                Exit Sub
        End Select
    Else
        Select Case KeyCode
            Case vbKeyUp:       RaiseEvent TelnetReply(Chr(27) + "[A")
            Case vbKeyDown:     RaiseEvent TelnetReply(Chr(27) + "[B")
            Case vbKeyRight:    RaiseEvent TelnetReply(Chr(27) + "[C")
            Case vbKeyLeft:     RaiseEvent TelnetReply(Chr(27) + "[D")
            Case vbKeyHome:     RaiseEvent TelnetReply(Chr(27) + "[1")
            Case vbKeyEnd:      RaiseEvent TelnetReply(Chr(27) + "[2")
            Case vbKeyPageUp:   RaiseEvent TelnetReply(Chr(27) + "[5")
            Case vbKeyPageDown: RaiseEvent TelnetReply(Chr(27) + "[6")
            Case Else
                Exit Sub
        End Select
    End If
    KeyCode = 0
End Sub


